import { SelectQueryBuilder } from "typeorm";
/**
 * Pagination utility.
 *
 * @author Jeongho Nam - https://github.com/samchon
 */
export declare namespace Paginator {
    /**
     * A page.
     *
     * Collection of records with pagination indformation.
     */
    interface IPage<T extends object> {
        /**
         * Page information.
         */
        pagination: IPagination;
        /**
         * List of records.
         */
        data: T[];
    }
    /**
     * Page information.
     */
    interface IPagination {
        /**
         * Current page number.
         */
        page: number;
        /**
         * Limitation of records per a page.
         *
         * @default 100
         */
        limit: number;
        /**
         * Total records in the database.
         */
        total_count: number;
        /**
         * Total pages.
         *
         * Equal to {@link total_count} / {@link limit} with ceiling.
         */
        total_pages: number;
    }
    /**
     * Page request data
     */
    interface IRequest {
        /**
         * Page number.
         */
        page?: number;
        /**
         * Limitation of records per a page.
         */
        limit?: number;
    }
    /**
     * Type of transformer.
     *
     * @param records Records to be transformed
     * @returns Transformed records
     */
    type Transformer<Input extends object, Output extends object> = (records: Input[]) => Output[] | Promise<Output[]>;
    /**
     * Pagination from regular ORM entity instances with transformation.
     *
     * Paginate regular ORM entity instances by getting from
     * {@link SelectQueryBuilder.getMany} method and transforming them to *Output* type.
     *
     * If you want to paginate not regular data, but raw data getting by
     * {@link SelectQueryBuilder.getRawMany}, use {@link rough} or {@link raw}
     * function instead. For referecen, different between {@link rough} and {@link raw}
     * is whether do transform or not.
     *
     * @template Entity Target entity type
     * @param stmt Select query builder for Entity type
     * @returns Currying function
     */
    const regular: <Entity extends object>(stmt: SelectQueryBuilder<Entity>) => <Output extends object>(transformer: Transformer<Entity, Output>) => (request: IRequest) => Promise<IPage<Output>>;
    /**
     * Pagination from raw data with transformation.
     *
     * Paginate raw data by getting from {@link SelectQueryBuilder.getRawMany}
     * method and transforming them to *Output* type.
     *
     * When you want to paginate regular ORM entity instances, use {@link regular}
     * function instead. Otherwise, you do not need transformation, just use
     * {@link raw} function.
     *
     * @template Entity Target entity type
     * @param stmt Select query builder for Entity type
     * @returns Currying function
     */
    const rough: <Entity extends object>(stmt: SelectQueryBuilder<Entity>) => <Raw extends object, Output extends object>(transformer: Transformer<Raw, Output>) => (request: IRequest) => Promise<IPage<Output>>;
    /**
     * Pagination from raw data without transformation.
     *
     * Paginate raw data by getting from {@link SelectQueryBuilder.getRawMany}
     * method. Unlink {@link rough}, this function does not transform raw data.
     * Therefore, the *Output* type must be constructed only by raw query.
     *
     * Otherwise you want to transform raw data, use {@link rough} function
     * instead. On contrary, if you want to paginate regular ORM entity instances,
     * use {@link regular} function instead.
     *
     * @template Entity Target entity type
     * @param stmt Select query builder for Entity type
     * @returns Currying function
     */
    const raw: <Entity extends object>(stmt: SelectQueryBuilder<Entity>) => <Output extends object>(request: IRequest) => Promise<IPage<Output>>;
    /**
     * Add order by statements to query builder.
     *
     * @param stmt Target select query builder
     * @param fieldList List of fields to order with direction
     * @param dictionary Dictionary for real column names if required
     * @param groupped Whether be groupped or not
     * @return Query builder itself
     */
    function orderBy<T, Field extends string, Sortable extends Array<`-${Field}` | `+${Field}`>>(stmt: SelectQueryBuilder<T>, fieldList: Sortable, dictionary?: Record<Field, string | (() => string)>, groupped?: boolean): SelectQueryBuilder<T>;
    /**
     * Follow order of source.
     *
     * Sort target array by following the order of source array.
     *
     * @param target Target array to sort
     * @param source Source array to be referenced
     * @param keyGetter Key getter function from any record
     * @return Target array itself
     */
    function follow<Target extends {
        id: Type;
    }, Source extends {
        id: Type;
    }, Type extends string | number>(target: Target[], source: Source[], keyGetter?: (elem: Target) => string | number | null): Target[];
    const DEFAULT_LIMIT = 100;
}
